---------------------------------------------------------------------------
--                                                                       --
--  ast2dot is an ASIS application that outputs the AST in dot format.   --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--        Copyright (c) 2006,2009 Raul Fechete, Georg Kienesberger       --
--           Copyright (c) 2000, Free Software Foundation, Inc.          --
--                                                                       --
---------------------------------------------------------------------------
-- This file is part of ast2dot, and is based on the ASIS Application    --
-- Templates. Date of change: 20 December 2006                           --
--                                                                       --
-- ast2dot is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 2 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- ast2dot is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with ast2dot; if not, write to the Free Software                --
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 --
-- USA                                                                   --
--                                                                       --
---------------------------------------------------------------------------

with Asis; use Asis;

with Ada.Wide_Text_IO;
with Ada.Characters.Handling;
with Ada.Exceptions;

with Asis.Exceptions;
with Asis.Errors;
with Asis.Implementation;

with Asis.Elements;

with Transformation_Control; use Transformation_Control;
with Process_Output; use Process_Output;

separate (Actuals_For_Traversing)
procedure Pre_Op
  (Element :        Asis.Element;
   Control : in out Asis.Traverse_Control;
   State   : in out Traversal_State)
is
begin

   IncLevel;
   PrintDot (MyElement=>Element, MyLevel=>GetLevel, MyLastLevel=>GetLastLevel, MyId=>GetId, MyLevelList=>LevList);

exception

   when Ex : Asis.Exceptions.ASIS_Inappropriate_Context          |
             Asis.Exceptions.ASIS_Inappropriate_Container        |
             Asis.Exceptions.ASIS_Inappropriate_Compilation_Unit |
             Asis.Exceptions.ASIS_Inappropriate_Element          |
             Asis.Exceptions.ASIS_Inappropriate_Line             |
             Asis.Exceptions.ASIS_Inappropriate_Line_Number      |
             Asis.Exceptions.ASIS_Failed                         =>

      Ada.Wide_Text_IO.Set_Output(Ada.Wide_Text_IO.Standard_Error);

      Ada.Wide_Text_IO.Put ("Pre_Op : ASIS exception (");

      Ada.Wide_Text_IO.Put (Ada.Characters.Handling.To_Wide_String (
              Ada.Exceptions.Exception_Name (Ex)));

      Ada.Wide_Text_IO.Put (") is raised");
      Ada.Wide_Text_IO.New_Line;

      Ada.Wide_Text_IO.Put ("ASIS Error Status is ");

      Ada.Wide_Text_IO.Put
        (Asis.Errors.Error_Kinds'Wide_Image (Asis.Implementation.Status));

      Ada.Wide_Text_IO.New_Line;

      Ada.Wide_Text_IO.Put ("ASIS Diagnosis is ");
      Ada.Wide_Text_IO.New_Line;
      Ada.Wide_Text_IO.Put (Asis.Implementation.Diagnosis);
      Ada.Wide_Text_IO.New_Line;

      Asis.Implementation.Set_Status;

      Ada.Wide_Text_IO.Set_Output(Ada.Wide_Text_IO.Standard_Output);

   when Ex : others =>

      Ada.Wide_Text_IO.Set_Output(Ada.Wide_Text_IO.Standard_Error);

      Ada.Wide_Text_IO.Put ("Pre_Op : ");

      Ada.Wide_Text_IO.Put (Ada.Characters.Handling.To_Wide_String (
              Ada.Exceptions.Exception_Name (Ex)));

      Ada.Wide_Text_IO.Put (" is raised (");

      Ada.Wide_Text_IO.Put (Ada.Characters.Handling.To_Wide_String (
              Ada.Exceptions.Exception_Information (Ex)));

      Ada.Wide_Text_IO.Put (")");
      Ada.Wide_Text_IO.New_Line;

      Ada.Wide_Text_IO.Set_Output(Ada.Wide_Text_IO.Standard_Output);

end Pre_Op;
